@@ -23,9 +23,10 @@
 #' Relevant if some data points are excluded, e.g. when using \code{\link{nndm}}.
 #' @param method Character. Method used for distance calculation. Currently euclidean distance (L2) and Mahalanobis distance (MD) are implemented but only L2 is tested. Note that MD takes considerably longer.
 #' @param useWeight Logical. Only if a model is given. Weight variables according to importance in the model?
+#' @param useCV Logical. Only if a model is given. Use the CV folds to calculate the DI threshold?
 #' @param LPD Logical. Indicates whether the local point density should be calculated or not.
 #' @param verbose Logical. Print progress or not?
-#'
+#' @param algorithm see \code{\link[FNN]{knnx.dist}} and \code{\link[FNN]{knnx.index}}
 #' @seealso \code{\link{aoa}}
 #' @importFrom graphics boxplot
 #' @import ggplot2
@@ -101,8 +102,10 @@ trainDI <- function(model = NA,
                     CVtrain = NULL,
                     method="L2",
                     useWeight = TRUE,
+                    useCV =TRUE,
                     LPD = FALSE,
-                    verbose = TRUE){
+                    verbose = TRUE,
+                    algorithm = "brute"){
 
   # get parameters if they are not provided in function call-----
   if(is.null(train)){train = aoa_get_train(model)}
@@ -127,7 +130,7 @@ trainDI <- function(model = NA,
   }
 
   # get CV folds from model or from parameters
-  folds <-  aoa_get_folds(model,CVtrain,CVtest)
+  folds <-  aoa_get_folds(model,CVtrain,CVtest,useCV)
   CVtest <- folds[[2]]
   CVtrain <- folds[[1]]
 
@@ -192,7 +195,7 @@ trainDI <- function(model = NA,
     #trainDist_avrg <- append(trainDist_avrg, mean(trainDistAll, na.rm = TRUE))
 
     # calculate  distance to other training data:
-    trainDist      <- matrix(.alldistfun(t(matrix(train[i,])), train, method, sorted = FALSE, S_inv))
+    trainDist      <- matrix(.alldistfun(t(matrix(train[i,])), train, method, sorted = FALSE, S_inv,algorithm=algorithm))
     trainDist[i]   <- NA
     trainDist_avrg <- append(trainDist_avrg, mean(trainDist, na.rm = TRUE))
 
@@ -259,7 +262,7 @@ trainDI <- function(model = NA,
     for (j in  seq(nrow(train))) {
 
       # calculate  distance to other training data:
-      trainDist      <- .alldistfun(t(matrix(train[j,])), train, method, sorted = FALSE, S_inv)
+      trainDist      <- .alldistfun(t(matrix(train[j,])), train, method, sorted = FALSE, S_inv,algorithm=algorithm)
       DItrainDist <- trainDist/trainDist_avrgmean
       DItrainDist[j]   <- NA
 
@@ -394,6 +397,9 @@ aoa_get_weights = function(model, variables){
       message("negative weights were set to 0")
     }
   }
+  if(sum(weight)==0){
+    stop("all weights are <=0, hence no variable is used. Check variable importance of the model, define weights manually or set useWeight=FALSE")
+  }
   return(weight)
 
 }
@@ -446,9 +452,9 @@ aoa_get_train <- function(model){
 # Get folds from train object
 
 
-aoa_get_folds <- function(model, CVtrain, CVtest){
+aoa_get_folds <- function(model, CVtrain, CVtest, useCV){
   ### if folds are to be extracted from the model:
-  if (!is.na(model)[1]){
+  if (useCV&!is.na(model)[1]){
     if(tolower(model$control$method)!="cv"){
       message("note: Either no model was given or no CV was used for model training. The DI threshold is therefore based on all training data")
     }else{
@@ -478,6 +484,12 @@ aoa_get_folds <- function(model, CVtrain, CVtest){
         }
       }
     }
+
+  }
+  if(!is.na(model)[1]&useCV==FALSE){
+    message("note: useCV is set to FALSE. The DI threshold is therefore based on all training data")
+    CVtrain <- NULL
+    CVtest <- NULL
   }
   return(list(CVtrain,CVtest))
 }
@@ -507,10 +519,10 @@ aoa_get_variables <- function(variables, model, train){
 
 
 
-.mindistfun <- function(point, reference, method, S_inv=NULL){
+.mindistfun <- function(point, reference, method, S_inv=NULL,algorithm){
 
   if (method == "L2"){ # Euclidean Distance
-    return(c(FNN::knnx.dist(reference, point, k = 1)))
+    return(c(FNN::knnx.dist(reference, point, k = 1, algorithm = algorithm)))
   } else if (method == "MD"){ # Mahalanobis Distance
     return(sapply(1:dim(point)[1],
                   function(y) min(sapply(1:dim(reference)[1],
@@ -518,13 +530,13 @@ aoa_get_variables <- function(variables, model, train){
   }
 }
 
-.alldistfun <- function(point, reference, method, sorted = TRUE,S_inv=NULL){
+.alldistfun <- function(point, reference, method, sorted = TRUE,S_inv=NULL,algorithm){
 
   if (method == "L2"){ # Euclidean Distance
     if(sorted){
-      return(FNN::knnx.dist(reference, point, k = dim(reference)[1]))
+      return(FNN::knnx.dist(reference, point, k = dim(reference)[1], algorithm = algorithm))
     } else {
-      return(FNN::knnx.dist(point,reference,k=1))
+      return(FNN::knnx.dist(point,reference,k=1, algorithm=algorithm))
     }
   } else if (method == "MD"){ # Mahalanobis Distance
     if(sorted){
